require 'posix'

-- config for dfbterm
no_theme=1
no_cursor=1
font_size=20
width=70
height=25
joystick=1
scrollback=0
pos_x=40
pos_y=20
font_file="/usr/share/fonts/truetype/Misc-Fixed.pfa"

-- menu configuration
-- where is the "dialog" utility? (executed via shell, thus PATH is effective)
sys_dialog="dialog"

-- gnuboy:
-- where to start browsing for roms:
gnuboy_romfilepath="/mnt/roms-gnuboy"
-- startup configuration
gnuboy_cmd="/usr/bin/dfbesdgnuboy --savedir=/tmp --bind joy8 quit --scale=3 --density=3 "

-- joystick mapping daemon:
js_host="127.0.0.1"
js_port=9021

-- joystick mappings:
jsmap_unmap = [[
unmapbuttons
]]
--   mappings know \n (enter), \t (tab), \s==" " (space), \e==\27 (escape)
jsmap_default = [[
unmapbuttons
mapbutton 0 \n
mapbutton 1 \s
mapbutton 2 \n
mapbutton 3 \s
mapaxis 1 - \e[A
mapaxis 1 + \e[B
mapaxis 0 + \e[C
mapaxis 0 - \e[D
]]

jsmap_cursor = [[
unmapbuttons
mapbutton 0 \n
mapbutton 1 \s
mapbutton 2 \n
mapbutton 3 \s
mapaxis 1 - \eOA
mapaxis 1 + \eOB
mapaxis 0 + \eOC
mapaxis 0 - \eOD
]]

-- the following function sends data to the joystick mapping daemon:
do
	-- we keep the socket as a kind of static (the dfbterm process fork()s after reading this file)
	setmap_socket=nil
	function jsd_setmap(btnmap)
		if not setmap_socket then
			-- initialize socket
			setmap_socket=require("socket")
		end
		if setmap_socket then
			c = setmap_socket.connect(js_host, js_port)
			c:send(btnmap)
			c:shutdown()
			c:close()
		end
	end
end

-- this is the corresponding handler for dfbterm:
do
	mapsocket=nil
	function map_jsinput(type, device_id, flags, button, buttons, axis, axisabs, axisrel, vt_mode)
		if not mapsocket then
			mapsocket=require("socket")
		end
		if mapsocket and type == DIET_BUTTONPRESS then
			-- we have a buttonpress, so connect to the mapping daemon
			c = mapsocket.connect(js_host, js_port)
			-- send the command
			c:send("buttonpress ")
			c:send(button)
			c:send("\n")
			-- wait for reply
			mapstr, e = c:receive("*a")
			-- close connection
			c:shutdown()
			c:close()
			if not e and mapstr then
				-- a mapstr was returned, output it to the PTY
				vt_writechild(mapstr)
			end
		elseif mapsocket and type == DIET_AXISMOTION then
			-- we have a buttonpress, so connect to the mapping daemon
			c = mapsocket.connect(js_host, js_port)
			-- send the command
			c:send("axismotion ")
			c:send(axis)
			c:send(" ")
			c:send(axisabs)
			c:send("\n")
			-- wait for reply
			mapstr, e = c:receive("*a")
			-- close connection
			c:shutdown()
			c:close()
			if not e and mapstr then
				-- a mapstr was returned, output it to the PTY
				vt_writechild(mapstr)
			end
		end 
	end
end


-- this is a wrapper for "dialog", which will be spawned in a subshell
-- returns three values:
--  a string status, which is "exited" if everything went alright
--  a number indicating the return value of dialog
--  a string containing the output of dialog (result value)
function call_dialog(...)
	-- set up a pair of pipes to communicate back the result
	pipe_read, pipe_write = posix.pipe()
	pid=posix.fork()
	if pid == -1 then
		io.stderr:write("error calling dialog...\n")
		posix.sleep(5)
		return "error", nil, nil
	elseif pid == 0 then
		-- child
		-- close reading side of the pipe
		pipe_read:close()
		-- execvp() to the dialog utility, passing a configured "--output-fd" to it
		posix.execp(sys_dialog, "--output-fd", posix.fileno(pipe_write), unpack(arg))
	else
		-- parent
		-- set up cursor mappings for joystick:
		jsd_setmap(jsmap_cursor)
		-- close the write side of the pipe
		pipe_write:close()
		-- now read (i.e. wait) for a result value
		result = pipe_read:read("*a")
		-- close our reading end of the pipe
		pipe_read:close()
		-- wait() for the child to clean up process space
		pid, status, retval = posix.wait(pid)
		-- reset default joystick mapping
		jsd_setmap(jsmap_default)
		-- return results etc
		return status, retval, result
	end
end

function main_menu()
	status, retval, result = call_dialog("--menu", "Select task:", 18, 52, 11, 1, "Run GnuBoy...")
	if status=="exited" and result=="1" then
		task_gnuboy()
		return 1
	end
	return 0
end

function task_gnuboy()
	local romfile=gnuboy_romfilepath
	repeat
		status, retval, romfile = call_dialog("--fselect", romfile.."/", 14, 52)
		if status=="exited" and retval==0 then
			-- we have a result, stat() it
			filestat = posix.stat(romfile)
		elseif retval~=0 then
			-- user chose to cancel the dialog
			return
		end
	until status=="exited" and retval==0 and filestat.type~="directory" and posix.access(romfile,"f")
	-- unset joystick/keyboard mapping, since the joystick mapper
	-- will happily continue to send events to the terminal otherwise
	jsd_setmap(jsmap_unmap)
	-- now we start gnuboy...
	os.execute(gnuboy_cmd.." '"..romfile.."'")
	-- when it finishes, we return here. We put up the default joystick map again...
	jsd_setmap(jsmap_default)
end

function lua_exec()
	local run=1
	while run ~= 0 do
		run = main_menu()
	end
	return 0
end
